
import logging

from airless.config import get_config


class BaseHook():

    def __init__(self):
        self.logger = logging.getLogger(f'{self.__class__.__module__}.{self.__class__.__name__}')
        logging.basicConfig(level=logging.getLevelName(get_config('LOG_LEVEL')))
        self.logger.debug(f'Created class instance {self.__class__.__name__}')




from airless.operator.base import BaseEventOperator
from airless.hook.notification.slack import SlackHook


class SlackSendOperator(BaseEventOperator):

    def __init__(self):
        super().__init__()
        self.slack_hook = SlackHook()

    def execute(self, data, topic):
        channels = data['channels']
        message = data['message']

        for channel in channels:
            self.slack_hook.send(channel, message)




import json

from google.cloud import secretmanager


class SecretManagerHook():

    def __init__(self):
        self.client = secretmanager.SecretManagerServiceClient()

    def list_secrets(self, project_id):
        request = {
            'parent': f'projects/{project_id}'
        }
        return [secret.name.split('/')[-1] for secret in self.client.list_secrets(request=request)]

    def list_secret_versions(self, project_id, secret_name, filter='state:(ENABLED OR DISABLED)'):
        request = {
            'parent': self.client.secret_path(project_id, secret_name),
            'filter': filter
        }

        return [version.name.split('/')[-1] for version in self.client.list_secret_versions(request=request)]

    def destroy_secret_version(self, project_id, secret_name, version):
        request = {
            'name': f'projects/{project_id}/secrets/{secret_name}/versions/{version}'
        }
        response = self.client.destroy_secret_version(request=request)

        return response.name

    def get_secret(self, project, id, parse_json=False):
        name = f'projects/{project}/secrets/{id}/versions/latest'
        response = self.client.access_secret_version(request={'name': name})
        decoded_response = response.payload.data.decode("UTF-8")

        if parse_json:
            return json.loads(decoded_response)
        else:
            return decoded_response

    def add_secret_version(self, project, id, value):
        parent = self.client.secret_path(project, id)
        payload = json.dumps(value) if isinstance(value, dict) else value
        response = self.client.add_secret_version(
            request={'parent': parent, 'payload': {"data": payload.encode('UTF-8')}}
        )
        return response




import json
import logging
import re
import time
import traceback

from base64 import b64decode

from airless.config import get_config
from airless.hook.google.pubsub import PubsubHook


class BaseOperator():

    def __init__(self):
        self.logger = logging.getLogger(f'{self.__class__.__module__}.{self.__class__.__name__}')
        logging.basicConfig(level=logging.getLevelName(get_config('LOG_LEVEL')))
        logging.debug(f'Created class instance {self.__class__.__name__}')
        self.pubsub_hook = PubsubHook()
        self.trigger_type = None
        self.message_id = None

    def extract_message_id(self, cloud_event):
        return cloud_event['id']

    def report_error(self, message):
        if get_config('ENV') == 'prod':
            logging.error(f'Error {message}')
        else:
            logging.debug(f'[DEV] Error {message}')

        error_obj = self.build_error_message(message)
        self.pubsub_hook.publish(
            project=get_config('GCP_PROJECT'),
            topic=get_config('PUBSUB_TOPIC_ERROR'),
            data=error_obj)

    def build_error_message(self, message):
        raise NotImplementedError()


class BaseFileOperator(BaseOperator):

    def __init__(self):
        super().__init__()

        self.trigger_type = 'file'
        self.trigger_origin = None
        self.cloud_event = None

    def execute(self, bucket, filepath):
        raise NotImplementedError()

    def run(self, cloud_event):
        try:
            self.message_id = self.extract_message_id(cloud_event)
            self.cloud_event = cloud_event
            trigger_file_bucket = cloud_event['bucket']
            trigger_file_path = re.sub(r'^objects\/', '', cloud_event['subject'])
            self.trigger_origin = f'{trigger_file_bucket}/{trigger_file_path}'
            self.execute(trigger_file_bucket, trigger_file_path)

        except Exception as e:
            self.report_error(f'{str(e)}\n{traceback.format_exc()}')

    def build_error_message(self, message):
        return {
            'input_type': self.trigger_type,
            'origin': self.trigger_origin,
            'error': message,
            'event_id': self.message_id,
            'data': {
                'attributes': self.cloud_event._attributes,
                'data': self.cloud_event.data
            }
        }


class BaseEventOperator(BaseOperator):

    def __init__(self):
        super().__init__()

        self.trigger_type = 'event'
        self.trigger_event_topic = None
        self.trigger_event_data = None

    def execute(self, data, topic):
        raise NotImplementedError()

    def run(self, cloud_event):
        try:
            self.message_id = self.extract_message_id(cloud_event)
            decoded_data = b64decode(cloud_event.data['message']['data']).decode('utf-8')
            self.trigger_event_data = json.loads(decoded_data)
            self.trigger_event_topic = cloud_event['source'].split('/')[-1]

            self.execute(self.trigger_event_data, self.trigger_event_topic)

            tasks = self.trigger_event_data.get('metadata', {}).get('run_next', [])
            self.run_next(tasks)

        except Exception as e:
            self.report_error(f'{str(e)}\n{traceback.format_exc()}')

    def run_next(self, tasks):
        if tasks:
            time.sleep(10)
        for t in tasks:
            self.pubsub_hook.publish(
                project=t.get('project', get_config('GCP_PROJECT')),
                topic=t['topic'],
                data=t['data'])

    def build_error_message(self, message):
        return {
            'input_type': self.trigger_type,
            'origin': self.trigger_event_topic,
            'error': message,
            'event_id': self.message_id,
            'data': self.trigger_event_data
        }




from airless.operator.base import BaseEventOperator

from airless.config import get_config


class RedirectOperator(BaseEventOperator):

    """
    Operator that receives one event from a pubsub topic
    and publish multiple messages to another topic.

    It can receive 4 parameters:
    project: the project where the destination pubsub is hosted
    topic: the pubsub topic it must publish the newly generated messages
    messages: a list of messages to publish the topic
    params: a list of dicts containing a key and a list of values

    The output messages will be the product of messages and every param values list
    """

    def __init__(self):
        super().__init__()

    def execute(self, data, topic):
        to_project = data.get('project', get_config('GCP_PROJECT'))
        to_topic = data['topic']
        messages = data.get('messages', [{}])
        params = data.get('params', [])

        messages = self.add_params_to_messages(messages, params)

        for msg in messages:
            self.pubsub_hook.publish(to_project, to_topic, msg)

    def add_params_to_messages(self, messages, params):
        for param in params:
            messages = self.add_param_to_messages(messages, param)
        return messages

    def add_param_to_messages(self, messages, param):
        messages_with_param = []
        for message in messages:
            messages_with_param += self.add_param_to_message(message, param)
        return messages_with_param

    def add_param_to_message(self, message, param):
        messages = []
        for value in param['values']:
            tmp_message = message.copy()
            keys = param['key'].split('.')
            tmp_message = self.add_key(tmp_message, keys, value)
            messages.append(tmp_message)
        return messages

    def add_key(self, obj, keys, value):
        tmp_obj = obj.copy()
        if len(keys) == 1:
            tmp_obj[keys[0]] = value
        else:
            nested_obj = tmp_obj.setdefault(keys[0], {})
            tmp_obj[keys[0]] = self.add_key(nested_obj, keys[1:], value)
        return tmp_obj




import smtplib

from email import encoders
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication

from airless.hook.base import BaseHook
from airless.hook.google.secret_manager import SecretManagerHook

from airless.config import get_config


class EmailHook(BaseHook):

    def __init__(self):
        super().__init__()
        secret_manager_hook = SecretManagerHook()
        self.smtp = secret_manager_hook.get_secret(get_config('GCP_PROJECT'), 'smtp', parse_json=True)

    def build_message(self, subject, content, recipients, sender, attachments=[]):
        msg = MIMEText(content)
        if attachments:
            msg = MIMEMultipart()
        msg['Subject'] = subject
        msg['To'] = ','.join(recipients)
        msg['From'] = sender

        for att in attachments:
            if att.get('type', 'text') == 'text':
                part = MIMEApplication(
                    att['content'],
                    Name=att['name']
                )
            else:
                part = MIMEBase('application', 'octet-stream')
                part.set_payload(att['content'])
                encoders.encode_base64(part)
            part['Content-Disposition'] = 'attachment; filename="%s"' % att['name']
            msg.attach(part)
        return msg

    def send(self, subject, content, recipients, sender, attachments):

        msg = self.build_message(subject, content, recipients, sender, attachments)
        server = smtplib.SMTP_SSL(self.smtp['host'], self.smtp['port'])

        try:
            server.login(self.smtp['user'], self.smtp['password'])
            server.sendmail(sender, recipients, msg.as_string())
        finally:
            server.close()
